SystemOrganization addCategory: #GODTextIO!

Object subclass: #TXTIOConverter
	instanceVariableNames: 'types'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODTextIO'!

!TXTIOConverter commentStamp: 'jlcg 10/17/2014 18:29' prior: 0!
Text input and output interface with GOD.!

!TXTIOConverter methodsFor: 'accessing' stamp: 'jlcg 10/31/2014 17:54'!
getFileTypeFromPath: aPath
	"get type (String) of the file from path"

	| parts |

	parts := aPath subStrings: '.'.
	^parts atWrap: parts size.! !

!TXTIOConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 21:13'!
getPossiblesTypes
	"get the possibles types to processing"

	^ types keys.! !

!TXTIOConverter methodsFor: 'initialize-release' stamp: 'jlcg 11/17/2014 15:51'!
initialize
	"set the types dictionary"

	types := Dictionary new.

	types at: #rtf put: TXTRtfConverter;
                at: #odt put: TXTOdtConverter;
		    at: #pdf put: TXTPdfConverter;
		    at: #txt  put: TXTTextConverter.! !

!TXTIOConverter methodsFor: 'file reading' stamp: 'jlcg 10/17/2014 21:02'!
readFromFile: aPath
	"Read a file from path and converts its contents to a GODData object. Try to extract the file type from the path."

	| fileType |

	fileType := self getFileTypeFromPath: aPath.

	^ self readFromFile: aPath type: fileType.! !

!TXTIOConverter methodsFor: 'file reading' stamp: 'jlcg 11/18/2014 17:19'!
readFromFile: aPath type: fileType
	"Read a file of a given type from path and converts its contents to a GODData object."

	| converter godData |

	( types includesKey: fileType )
		ifFalse: [ GODTextIOInvalidFileType signal ].

	converter := (types at: fileType) new.
	converter path: aPath.
	converter readFile.

	godData := GODData new.
	godData timestamp: DateAndTime now.
	godData origin: fileType.
	godData content: converter contents.

	^godData.
! !

!TXTIOConverter methodsFor: 'file writing' stamp: 'jlcg 10/17/2014 21:03'!
write: godData toFile: aPath
	"Write a GODData object contents to a file. Try to extract the file type from its path."

	| fileType |

	fileType := self getFileTypeFromPath: aPath.

	^ self write: godData toFile: aPath type: fileType append: false overwrite: false.! !

!TXTIOConverter methodsFor: 'file writing' stamp: 'jlcg 10/17/2014 21:11'!
write: godData toFile: aPath type: fileType
	"Write a GODData object contents to a file of a given type."

	^ self write: godData toFile: aPath type: fileType append: false overwrite: false.! !

!TXTIOConverter methodsFor: 'file writing' stamp: 'jlcg 10/17/2014 21:11'!
write: godData toFile: aPath type: fileType append: aBoolean
	"Write a GODData object contents to a file of a given type. Append determines whether to append contents to an existing file on the given path."

	^ self write: godData toFile: aPath type: fileType append: aBoolean overwrite: false.! !

!TXTIOConverter methodsFor: 'file writing' stamp: 'jlcg 10/30/2014 21:09'!
write: godData toFile: aPath type: fileType append: appendBoolean overwrite: overwriteBoolean
	"Write contents of GODData object to a file."

	| converter |

	( types includesKey: fileType )
		ifFalse: [ GODTextIOInvalidFileType signal ].

	converter := (types at: fileType) new.
	converter append: appendBoolean.
	converter author: godData author.
	converter contents: godData content.
	converter date: godData timestamp.
	converter overwrite: overwriteBoolean.
	converter path: aPath.
	converter title: godData title.

	converter writeFile.

	^ true.! !

!TXTIOConverter methodsFor: 'file writing' stamp: 'jlcg 10/17/2014 21:12'!
write: godData toFile: aPath type: fileType overwrite: aBoolean
	"Write a GODData object contents to a file of a given type. Overwrite determines whether to overwrite or not an existing file of the same name in the given path."

	^ self write: godData toFile: aPath type: fileType append: false overwrite: aBoolean.! !

Object subclass: #TXTMainConverter
	instanceVariableNames: 'author append contents date overwrite path title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODTextIO'!

!TXTMainConverter commentStamp: 'jlcg 10/16/2014 11:27' prior: 0!
The main class of this module. It is an abstract class that models the main methods of the subclasses.!

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:45'!
append
	^ append.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:45'!
append: aBoolean

	append := aBoolean.
! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/30/2014 21:08'!
author
	^ author.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/30/2014 21:07'!
author: aString
	author := aString.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:45'!
contents
	^ contents! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:46'!
contents: aString
	contents := aString.! !

!TXTMainConverter methodsFor: 'file processing' stamp: 'jlcg 11/21/2014 17:38'!
convertFileType: fileType
	"Convert file in path to fileType using abiword."

	| tmpPath |

	tmpPath := ( FileDirectory default ) pathName, FileDirectory slash, DateAndTime now asNanoSeconds, '.', fileType.

	OSProcess waitForCommand: 'abiword --to=', fileType, ' --to-name=', tmpPath, ' ', path.

	( FileDirectory new fileExists: tmpPath )
		ifTrue: [ ^tmpPath ]
		ifFalse: [ GODTextIOFailedToConvertType signal. ].! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/30/2014 21:07'!
date
	^ date.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/30/2014 21:07'!
date: aString
	date := aString.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:46'!
overwrite
	^ overwrite! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:46'!
overwrite: aBoolean
	overwrite := aBoolean.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:46'!
path
	^ path.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/17/2014 18:47'!
path: aString
	path := aString.! !

!TXTMainConverter methodsFor: 'file processing' stamp: 'jlcg 11/19/2014 17:14'!
readFile
	"Basic methods that calls an automatic converter and read its contents."

	| textIO tmpFile |

	tmpFile := self convertFileType: 'odt'.

	textIO := TXTOdtConverter new.
	textIO path: tmpFile.
	textIO readFile.

	contents := textIO contents.

	( FileDirectory forFileName: tmpFile ) deleteFileNamed: tmpFile.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/30/2014 21:07'!
title
	^ title.! !

!TXTMainConverter methodsFor: 'accessing' stamp: 'jlcg 10/30/2014 21:07'!
title: aString
	title := aString.! !

!TXTMainConverter methodsFor: 'file processing' stamp: 'mso 10/7/2014 21:37'!
writeFile
	"abstract method"

	^ self subclassResponsibility! !

TXTMainConverter subclass: #TXTOdtConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODTextIO'!

!TXTOdtConverter commentStamp: 'jlcg 9/27/2014 08:34' prior: 0!
This class process ODF files. The development of this class is secondary.!

!TXTOdtConverter methodsFor: 'xml' stamp: 'jlcg 11/19/2014 17:21'!
extractTextXML
	"extract <office:text> block from ODT file"

	| odtContent text xmlCode |

	odtContent := [ ( ZipArchive new readFrom: path ) member: 'content.xml'. ] on: Error do: [ FileStream readOnlyFileNamed: path. ].
	xmlCode := XMLDOMParser parseDocumentFrom: odtContent contents readStream.

	text := [ ( ( xmlCode elementAt: #'office:document-content' ) 
						  elementAt: #'office:body' ) 
						  elementAt: #'office:text'. 
		      ] on: Error do: [ ( ( xmlCode elementAt: #'office:document' ) 
				 		  				elementAt: #'office:body' ) 
									  	elementAt: #'office:text'. 
						   ].

	odtContent close.

	^text elementsAndContents.
! !

!TXTOdtConverter methodsFor: 'util' stamp: 'jlcg 11/18/2014 17:36'!
odtContent
	"writes the content of contents.xml to a stream"

	| stream |

	stream := WriteStream on: ( String new ).

	^self odtContent: stream.! !

!TXTOdtConverter methodsFor: 'util' stamp: 'jlcg 12/2/2014 10:02'!
odtContent: aStream
	"writes the content of contents.xml to a stream"

	| cReadStream cWriteStream stream |

	cReadStream := ReadStream on: self contents.
	cWriteStream := WriteStream on: ( String new ).

	"This replacement is done in order to convert a line break in a different paragraph in ODT"
	cReadStream do: [ :each | 
						( each = Character cr )
							ifTrue: [ cWriteStream nextPutAll: '</text:p><text:p text:style-name="body">'. ]
							ifFalse: [ cWriteStream nextPut: each ].
					   ].

	stream := WriteStream on: ( String new ).
	stream nextPutAll: self odtHeader;
				nextPut: Character cr;
				nextPutAll: self odtStyle;
				nextPut: Character cr;
				nextPutAll:
'  <office:body>
    <office:text>
	', aStream contents, Character cr.

	( self title = nil )
		ifFalse: [ stream nextPutAll: '	<text:p text:style-name="title">', self title, '</text:p>', Character cr. ].

	( self author = nil )
		ifFalse: [ stream nextPutAll: '	<text:p text:style-name="author">', self author, '</text:p>', Character cr. ].

	( self date = nil )
		ifFalse: [ stream nextPutAll: '      <text:p text:style-name="date">', self date, '</text:p>', Character cr. ].

      stream nextPutAll: '
	<text:p text:style-name="body"/><text:p text:style-name="body">', cWriteStream contents, '</text:p>
    </office:text>
  </office:body>
</office:document-content>'.

	^stream contents.! !

!TXTOdtConverter methodsFor: 'util' stamp: 'jlcg 11/18/2014 10:42'!
odtHeader

^'<office:document-content xmlns:fo="urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0" xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0" xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0">'.! !

!TXTOdtConverter methodsFor: 'util' stamp: 'jlcg 11/18/2014 11:30'!
odtManifest

^'<?xml version="1.0" encoding="UTF-8"?>
<manifest:manifest xmlns:manifest="urn:oasis:names:tc:opendocument:xmlns:manifest:1.0" manifest:version="1.2">
 <manifest:file-entry manifest:full-path="/" manifest:version="1.2" manifest:media-type="application/vnd.oasis.opendocument.text"/>
 <manifest:file-entry manifest:full-path="content.xml" manifest:media-type="text/xml"/>
</manifest:manifest>'.! !

!TXTOdtConverter methodsFor: 'util' stamp: 'jlcg 11/18/2014 17:21'!
odtStyle

^'  <office:automatic-styles>
    <style:style style:name="title" style:family="paragraph">
      <style:paragraph-properties fo:text-align="center" style:justify-single-word="false"/>
      <style:text-properties style:font-name="Times New Roman" fo:font-size="16pt" fo:font-weight="bold" style:font-size-asian="16pt" style:font-size-complex="16pt"/>
    </style:style>
    <style:style style:name="author" style:family="paragraph">
      <style:paragraph-properties fo:text-align="center" style:justify-single-word="false"/>
      <style:text-properties style:font-name="Times New Roman" fo:font-size="14pt" style:font-size-asian="14pt" style:font-size-complex="14pt"/>
    </style:style>
    <style:style style:name="date" style:family="paragraph">
      <style:paragraph-properties fo:text-align="center" style:justify-single-word="false"/>
      <style:text-properties style:font-name="Times New Roman" fo:font-size="12pt" style:font-size-asian="12pt" style:font-size-complex="12pt"/>
    </style:style>
    <style:style style:name="body" style:family="paragraph">
      <style:text-properties style:font-name="Times New Roman" fo:font-size="11pt" style:font-size-asian="11pt" style:font-size-complex="11pt"/>
    </style:style>
  </office:automatic-styles>'.! !

!TXTOdtConverter methodsFor: 'file processing' stamp: 'jlcg 12/2/2014 11:13'!
readFile
	"Read contents of a ODT file in 'path' to 'content'."

	| contentsStream curElement tags |

	tags := self extractTextXML.

	contentsStream := WriteStream on: ( String new ).

	"Checks recursively all tags and extract only text to contentsStream"
	[ tags isEmpty ]
		whileFalse: [
			curElement := tags removeFirst.
			( curElement isText )
				ifTrue: [ contentsStream nextPutAll: curElement string. ]
				ifFalse: [ 
					tags addAllFirst: curElement elementsAndContents. 
					( ( ( curElement localName ) = 'text:p' ) and: ( ( contentsStream size ) > 0 ) )
						ifTrue: [ contentsStream nextPut: Character cr. ]. 
				].
		].

	contents := contentsStream contents.

	^true.! !

!TXTOdtConverter methodsFor: 'file processing' stamp: 'jlcg 11/18/2014 18:07'!
writeFile
	"Write 'content' to an ODT file in path 'path'."

	| appendStream odtFile metaInfDirectory textXML tmpDirectory |

	( append )
		ifTrue: [ ( FileDirectory new fileExists: path ) ifFalse: [ GODTextIOFileDoesNotExist signal ]. ]
		ifFalse: [
			( overwrite )
				ifFalse: [ ( FileDirectory new fileExists: path ) ifTrue: [ GODTextIOFileAlreadyExists signal. ]. ].
		].

	appendStream := WriteStream on: ( String new ).

	tmpDirectory := FileDirectory default / 'TEMP_TEXTIO'.
	tmpDirectory assureExistence.

	metaInfDirectory := tmpDirectory / 'META-INF'.
	metaInfDirectory assureExistence.

	"Create file manifest.xml"
	metaInfDirectory fileNamed: 'manifest.xml' do: [ :stream | stream nextPutAll: self odtManifest. ].

	"Create file content.xml. Append old file content if append is true"
	tmpDirectory fileNamed: 'content.xml' do: [ :stream |
		( append )
			ifTrue: [
				textXML := self extractTextXML.
				[ textXML isEmpty ]
					whileFalse: [
						appendStream nextPutAll: ( textXML at: 1 ) asString.
						textXML removeFirst.
					].
				appendStream nextPutAll: '<text:p text:style-name="body"/>';
								nextPut: Character cr.
			].
		stream nextPutAll: ( self odtContent: appendStream ).
	].

	"Create odt file"
	odtFile := ZipArchive new.
	odtFile addDirectory: metaInfDirectory fullName as: 'META-INF'.
	odtFile addFile: ( metaInfDirectory fullNameFor: 'manifest.xml' ) as: ( 'META-INF', FileDirectory slash, 'manifest.xml' ).
	odtFile addFile: ( tmpDirectory fullNameFor: 'content.xml' ) as: ( 'content.xml' ).
	odtFile writeToFileNamed: path.
	odtFile close.

	tmpDirectory recursiveDelete.

	^true.! !

TXTMainConverter subclass: #TXTPdfConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODTextIO'!

!TXTPdfConverter commentStamp: 'jlcg 9/27/2014 08:34' prior: 0!
This class process PDF files. The development of this class is secondary.!

!TXTPdfConverter methodsFor: 'util' stamp: 'jlcg 12/2/2014 09:44'!
latexHeader
	"Returns a basic latex header to generate a PDF."

^'\documentclass[11pt]{article}
\usepackage[brazil,english]{babel}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage[top=2cm,bottom=2cm,left=2cm,right=2cm]{geometry}

\parindent 0cm
\parskip 0.2cm'.! !

!TXTPdfConverter methodsFor: 'file processing' stamp: 'jlcg 11/19/2014 17:05'!
readFile
	"Read contents of a PDF file in 'path' to 'content'."

	super readFile.

	^true.! !

!TXTPdfConverter methodsFor: 'file processing' stamp: 'jlcg 12/2/2014 10:08'!
writeFile
	"Write 'content' to a PDF file in path 'path'. Uses Latex."

	| cReadStream cWriteStream file1 file2 latexCode localAuthor localDate localTitle shStatus texDir texFile texPath |

	( overwrite )
		ifTrue: [ ( FileDirectory new fileExists: path ) ifTrue: [ ( FileDirectory forFileName: path ) deleteFileNamed: path. ] ]
		ifFalse: [ ( FileDirectory new fileExists: path ) ifTrue: [ GODTextIOFileAlreadyExists signal. ] ].

	texDir := FileDirectory default / 'TEMP_TEXTIO'.
	texDir assureExistence.

	texPath := texDir pathName, FileDirectory slash.

	cReadStream := ReadStream on: ( self contents ).
	cWriteStream := WriteStream on: ( String new ).

	"This replacement is done in order to convert a line break in a different paragraph in Latex"
	cReadStream do: [ :each | cWriteStream nextPut: each. ( each = Character cr ) ifTrue: [ cWriteStream nextPut: Character cr. ]. ].

	( self author = nil ) ifTrue: [ localAuthor := ''. ] ifFalse: [ localAuthor := self author. ].
	( self date = nil ) ifTrue: [ localDate := ''. ] ifFalse: [ localDate := self date. ].
	( self title = nil ) ifTrue: [ localTitle := ''. ] ifFalse: [ localTitle := self title. ].

	latexCode := self latexHeader, Character cr.
	latexCode := latexCode, '\title{', localTitle, '}', Character cr, '\author{', localAuthor, '}', Character cr, '\date{', localDate, '}', Character cr.
	latexCode := latexCode, '\begin{document}', Character cr, '\maketitle', Character cr, cWriteStream contents, Character cr, '\end{document}'.

	"Write the .tex file using the TXTTextConverter class."
	texFile := TXTTextConverter new.
	texFile append: false.
	texFile contents: latexCode.
	texFile overwrite: true.
	texFile path: texPath, 'godpdfgen.tex'.
	texFile writeFile.

	"Runs pdflatex"
	shStatus := OSProcess waitForCommand: 'pdflatex -halt-on-error -output-directory=', texPath, ' ', texPath, 'godpdfgen.tex'.
	( shStatus exitStatus == 0 )
		ifFalse: [ GODTextIOPdfLatexError signal. ].

	"Move the generated PDF to the one required by the user."
	file1 := ( FileStream oldFileNamed: texPath, 'godpdfgen.pdf' ) binary.
	file2 := ( FileStream newFileNamed: path ) binary.
	( FileDirectory new ) copyFile: file1 toFile: file2.

	"Remove temporary files."
	texDir recursiveDelete.

	^true! !

TXTMainConverter subclass: #TXTRtfConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODTextIO'!

!TXTRtfConverter commentStamp: 'jlcg 9/27/2014 08:34' prior: 0!
This class process .doc files. The development of this class is secondary.!

!TXTRtfConverter methodsFor: 'file processing' stamp: 'jlcg 11/19/2014 17:05'!
readFile
	"Read contents of a RTF file in 'path' to 'content'."

	super readFile.

	^true.! !

!TXTRtfConverter methodsFor: 'util' stamp: 'jlcg 11/17/2014 14:46'!
rtfHeader
	"Returns a basic RTF header"

	^'\rtf1\ansi\deff0{\fonttbl{\f0\fswiss Arial;}{\f1\froman Times New Roman;}{\f2\ftech Symbol;}}'.! !

!TXTRtfConverter methodsFor: 'file processing' stamp: 'jlcg 12/2/2014 10:36'!
writeFile
	"Write 'content' to a RTF file in path 'path'."

	| cReadStream cWriteStream rtfFile rtfCode |

	cReadStream := ReadStream on: ( self contents ).
	cWriteStream := WriteStream on: ( String new ).

	"This replacement is done in order to convert a line break in a different paragraph"
	cReadStream do: [ :each | ( each = Character cr ) ifTrue: [ cWriteStream nextPutAll: '\par ' ] ifFalse: [ cWriteStream nextPut: each. ]. ].

	rtfCode := '{', self rtfHeader, Character cr.
	( self title = nil ) ifFalse: [ rtfCode := rtfCode, '{\qc \b \fs32 ', self title, '\par}', Character cr. ].
	( self author = nil ) ifFalse: [ rtfCode := rtfCode, '{\qc \fs28 ', self author, '\par}', Character cr. ].
	( self date = nil ) ifFalse: [ rtfCode := rtfCode, '{\qc \fs24 ', self date, '\par}', Character cr. ].
	rtfCode := rtfCode, '\par \qj \fs22 ', cWriteStream contents, '\par}'.

	"Write the .rtf file using the TXTTextConverter class."
	rtfFile := TXTTextConverter new.
	rtfFile append: false.
	rtfFile contents: rtfCode.
	rtfFile overwrite: self overwrite.
	rtfFile path: self path.
	rtfFile writeFile.

	^true.! !

TXTMainConverter subclass: #TXTTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODTextIO'!

!TXTTextConverter commentStamp: 'jlcg 9/27/2014 08:33' prior: 0!
This class process text files. The development of this class is prioritary.!

!TXTTextConverter methodsFor: 'file processing' stamp: 'jlcg 10/17/2014 20:45'!
readFile
	"Read contents to a text file in 'path' to 'content'."

	| file |

	file := [ FileStream readOnlyFileNamed: path ] on: FileStreamException do: [ :exception | GODTextIOFileDoesNotExist signal ].

	contents := file contents.

	file close.

	^true.! !

!TXTTextConverter methodsFor: 'file processing' stamp: 'jlcg 10/31/2014 19:00'!
writeFile
	"Write 'content' to a text file in path 'path'."

	| file |

	( overwrite )
		ifTrue: [ file := [ FileStream forceNewFileNamed: path ] 
							on: Error do: [ :exception | exception signal ] ]
		ifFalse: [ ( append )
					ifTrue: [ file := [ FileStream oldFileNamed: path ]
										on: FileDoesNotExistException do: [ :exception | GODTextIOFileDoesNotExist signal ].
							 file setToEnd ; cr. ]
					ifFalse: [ file := [ FileStream newFileNamed: path ]
										on: FileExistsException do: [ :exception | GODTextIOFileAlreadyExists signal ]
										on: Error do: [ :exception | exception signal ] ] ].
	file nextPutAll: contents.
	file close.

	^true.! !
